import re

import pytest

from phone_gen._generator import NumberGenerator, NumberGeneratorException, NumberGeneratorSyntaxException


def test_one():
    data = NumberGenerator(r'[9-0][\d]{4}').render()
    assert re.match(r'\d{5}', data)


def test_two():
    NumberGenerator(r'[0-9][\d\{]').render()


SYNTAX_ERROR_PATTERN = (
    r'[136-9]\d{7}',
    r'[136-9][\d{7}',
    r'[136-9]\d]{7}',
    r'[136-9][\d]{7,8}',
    r'[136-9][\d]{7:}',
    r'(4[\d]{8})|([1-9][\d]{7}',
    r'(4[\d]{8})|[1-9][\d]{7})',
    r'(4[\d]{8})|',
    r'|([1-9][\d]{7})',
    r'(4[\d]{8})|([1-][\d]{7})'
)


@pytest.mark.parametrize('pattern', SYNTAX_ERROR_PATTERN)
def test_syntax_error(pattern):
    with pytest.raises(NumberGeneratorSyntaxException):
        NumberGenerator(pattern).render()


@pytest.mark.parametrize('pattern', (r'[0-9][\d]{',))
def test_base_error(pattern):
    with pytest.raises(NumberGeneratorException):
        NumberGenerator(pattern).render()



import random

import phonenumbers
import pytest

from phone_gen import PhoneNumber, PhoneNumberNotFound
from phone_gen.patterns import PATTERNS


@pytest.mark.phonenumbers
@pytest.mark.parametrize('count', range(15))
@pytest.mark.parametrize('country', PATTERNS['data'].keys())
def test_patterns(country, count):
    phone_number = PhoneNumber(country)
    number = phone_number.get_number()
    num_obj = phonenumbers.parse(number, country)
    assert phonenumbers.is_valid_number_for_region(num_obj, country)


@pytest.mark.phonenumbers
@pytest.mark.parametrize('count', range(15))
@pytest.mark.parametrize('country', PATTERNS['data'].keys())
def test_national(country, count):
    phone_number = PhoneNumber(country)
    number = phone_number.get_national()
    num_obj = phonenumbers.parse(number, country)
    assert phonenumbers.is_valid_number_for_region(num_obj, country)


@pytest.mark.phonenumbers
@pytest.mark.parametrize('count', range(15))
@pytest.mark.parametrize('country', PATTERNS['data'].keys())
def test_mobile(country, count):
    phone_number = PhoneNumber(country)
    number = phone_number.get_mobile()
    num_obj = phonenumbers.parse(number, country)
    assert phonenumbers.is_valid_number_for_region(num_obj, country)


def test_info():
    phone_number = PhoneNumber('gb')
    assert phone_number.info().startswith('libphonenumber')


def test_get_code():
    country = random.choice(tuple(PATTERNS['data'].keys()))
    phone_number = PhoneNumber(country)
    assert phone_number.get_code() == PATTERNS['data'][country]['code']


def test_invalid_country():
    with pytest.raises(PhoneNumberNotFound):
        PhoneNumber('qwe')


def test_str_method():
    phone_number = PhoneNumber('GB')
    assert str(phone_number).startswith('<PhoneNumber(libphonenumber')



import random
import re
import string
from abc import ABCMeta, abstractmethod
from dataclasses import dataclass
from typing import List, Optional, Tuple, Dict, Final

from .alt_patterns import ALT_PATTERNS
from .country_name import COUNTRY_NAME
from .iso3 import ISO3
from .patterns import PATTERNS

SYSTEM_RANDOM = random.SystemRandom()
RANDINT = SYSTEM_RANDOM.randint
CHOICE = SYSTEM_RANDOM.choice
SAMPLE = SYSTEM_RANDOM.sample
SHUFFLE = SYSTEM_RANDOM.shuffle

BSLASH: Final[str] = "\\"
LBRACE: Final[str] = "{"
RBRACE: Final[str] = "}"
LSQB: Final[str] = "["
RSQB: Final[str] = "]"
LPAR: Final[str] = "("
RPAR: Final[str] = ")"
COLON: Final[str] = ":"
MINUS: Final[str] = "-"
VBAR: Final[str] = "|"

META_TOKENS: Tuple[str, ...] = (
    LSQB,
    RSQB,
    LBRACE,
    RBRACE,
    LPAR,
    RPAR,
    VBAR,
)

ALTERNATIVE_FILE: dict = {}


def clean_alt_patters() -> None:
    ALTERNATIVE_FILE.clear()


def load_alt_patters(patters: dict) -> None:
    clean_alt_patters()
    ALTERNATIVE_FILE.update(patters)


class NumberGeneratorException(Exception):
    """Base Exception"""


class NumberGeneratorSyntaxException(NumberGeneratorException):
    """Syntax Exception"""


class PhoneNumberNotFound(Exception):
    """Not found country"""


class StringNode(metaclass=ABCMeta):
    """The abstract class for all nodes"""

    @abstractmethod
    def render(self): ...  # pragma: no cover


@dataclass(frozen=True)
class Sequence:
    """Render a sequence of nodes from the template."""

    seq: List[StringNode]

    def render(self) -> str:
        return "".join(i.render() for i in self.seq)


@dataclass(frozen=True)
class SequenceOR(Sequence):
    def render(self):
        return self.seq[RANDINT(0, len(self.seq) - 1)].render()


@dataclass(repr=True)
class Literal(StringNode):
    chars: str

    def render(self) -> str:
        return self.chars


@dataclass(repr=True)
class CharacterSet(StringNode):
    chars: str
    start: int
    cnt: int

    def render(self) -> str:
        cnt = RANDINT(self.start, self.cnt) if self.start > -1 else self.cnt
        return "".join(self.chars[RANDINT(0, len(self.chars) - 1)] for _ in range(cnt))


class NumberGenerator:
    """
    Modified StringGenerator https://github.com/paul-wolf/strgen
    """

    _code = {"d": string.digits}

    def __init__(self, pattern: str):
        self._pattern = pattern
        self._index = -1
        self._seq = self._get_sequence()

    def _current(self) -> Optional[str]:
        return self._pattern[self._index] if self._index < len(self._pattern) else None

    def _next(self) -> Optional[str]:
        self._index += 1
        return self._current()

    def _lookahead(self) -> Optional[str]:
        return (
            self._pattern[self._index + 1]
            if self._index + 1 < len(self._pattern)
            else None
        )

    def _last(self) -> Optional[str]:
        return None if self._index == 0 else self._pattern[self._index - 1]

    def _get_quantifier(self) -> Tuple[int, int]:
        start = -1
        self._next()
        digits = "0"
        while True:
            d = self._next()
            if not d:
                raise NumberGeneratorException(
                    "Unexpected end of input getting quantifier"
                )
            if d in (COLON, MINUS):
                start = int(digits)
                digits = "0"
                continue
            if d == RBRACE:
                if self._last() in (COLON, MINUS):
                    raise NumberGeneratorSyntaxException(
                        "Quantifier range must be closed"
                    )
                break  # pragma: no cover
            if d.isnumeric():
                digits += d
            else:
                raise NumberGeneratorSyntaxException("Non-digit in count")
        return start, int(digits)

    @staticmethod
    def _get_character_range(first: str, last: str) -> str:
        chars = ""
        # support z-a as a range
        if not ord(first) < ord(last):
            first, last = last, first
        for c in range(ord(first), ord(last) + 1):
            chars += chr(c)
        return chars

    def _get_character_set(self) -> CharacterSet:
        chars = ""
        cnt = 1
        start = 0

        while True:
            char = self._next()
            if self._lookahead() == MINUS and not char == BSLASH:
                f = char
                self._next()  # skip hyphen
                char = self._next()  # get far range
                if not char or (char in META_TOKENS):
                    raise NumberGeneratorSyntaxException(
                        "unexpected end of class range"
                    )
                chars += self._get_character_range(f, char)
            elif char == BSLASH:
                if self._lookahead() in META_TOKENS:
                    char = self._next()
                    chars += char
                    continue
                elif self._lookahead() in self._code:
                    char = self._next()
                    chars += self._code[char]
            elif char and char not in META_TOKENS:
                chars += char
            if char == RSQB:
                if self._lookahead() == LBRACE:
                    start, cnt = self._get_quantifier()
                else:
                    start = -1
                    cnt = 1
                break
            if char and char in META_TOKENS and not self._last() == BSLASH:
                raise NumberGeneratorSyntaxException(
                    f"Un-escaped character in class definition: {char}"
                )
            if not char:
                break  # pragma: no cover
        return CharacterSet(chars=chars, start=start, cnt=cnt)

    def _get_literal(self) -> Literal:
        chars = ""
        char = self._current()
        while True:
            if char and char == BSLASH:
                char = self._next()
                if char:
                    chars += char
                continue
            elif not char or (char in META_TOKENS):
                break  # pragma: no cover
            else:
                chars += char
            if self._lookahead() and self._lookahead() in META_TOKENS:
                break
            char = self._next()
        return Literal(chars=chars)

    def _get_sequence(self, level: int = 0) -> Sequence:
        seq = []
        op = ""
        left_operand = None
        sequence_closed = False
        while True:
            char = self._next()
            if not char:
                break
            is_last_bslash = self._last() == BSLASH
            if char and char not in META_TOKENS:
                seq.append(self._get_literal())
            elif char == LSQB and not is_last_bslash:
                seq.append(self._get_character_set())
            elif char == LPAR and not is_last_bslash:
                seq.append(self._get_sequence(level + 1))
            elif char == RPAR and not is_last_bslash:
                # end of this sequence
                if level == 0:
                    # there should be no parens here
                    raise NumberGeneratorSyntaxException("Extra closing parenthesis")
                sequence_closed = True
                break
            elif char == VBAR and not is_last_bslash:
                op = char
            else:
                if char in META_TOKENS and not is_last_bslash:
                    raise NumberGeneratorSyntaxException(
                        f"Un-escaped special character: {char}"
                    )

            if op and not left_operand:
                if not seq:
                    raise NumberGeneratorSyntaxException(
                        f"Operator: {op} with no left operand"
                    )
                left_operand = seq.pop()
            elif op and len(seq) >= 1 and left_operand:
                right_operand = seq.pop()

                if op == VBAR:
                    seq.append(SequenceOR(seq=[left_operand, right_operand]))
                op = ""
                left_operand = None

        # check for syntax errors
        if op:
            raise NumberGeneratorSyntaxException(
                f"Operator: {op} with no right operand"
            )
        if level > 0 and not sequence_closed:
            raise NumberGeneratorSyntaxException("Missing closing parenthesis")
        return Sequence(seq=seq)

    def render(self) -> str:
        return self._seq.render()


class PhoneNumber:
    def __init__(self, value: str) -> None:
        code = self._preparation(value)
        self._country = self._find(code)
        if not self._country:
            raise PhoneNumberNotFound(f'Not found country "{value}"')

    def __str__(self):
        return f"<{type(self).__name__}({self.info()})>"

    def _find(self, value: str) -> Optional[Dict[str, str]]:
        if country := ALTERNATIVE_FILE.get(value) or PATTERNS["data"].get(value):
            return country
        if alt_country := ALT_PATTERNS.get(value):
            return (
                alt_country
                if "pattern" in alt_country
                else self._find(alt_country["ref"])
            )
        if country_name := COUNTRY_NAME.get(value):
            return self._find(country_name["code"])
        if ico3 := ISO3.get(value):
            return self._find(ico3["code"])
        return None

    @staticmethod
    def _preparation(value: str) -> str:
        return re.sub(r"\W", "", value).upper()

    @staticmethod
    def info() -> str:
        return PATTERNS["info"]

    def get_code(self) -> str:
        return self._country["code"]

    def get_number(self, full: bool = True) -> str:
        return self.get_national(full) if RANDINT(0, 1) else self.get_mobile(full)

    def get_mobile(self, full: bool = True) -> str:
        number = NumberGenerator(
            pattern=self._country.get("mobile", self._country["pattern"])
        ).render()
        return f"+{self._country['code']}{number}" if full else number

    def get_national(self, full: bool = True) -> str:
        number = NumberGenerator(pattern=self._country["pattern"]).render()
        # Could not find problem
        if (
            number.startswith("49") and self._country["code"] == "49"
        ):  # pragma: no cover
            return self.get_national(full)
        return f"+{self._country['code']}{number}" if full else number



import phonenumbers
import pytest

from phone_gen.alt_patterns import ALT_PATTERNS
from phone_gen import PhoneNumber


@pytest.mark.phonenumbers
@pytest.mark.parametrize('count', range(5))
@pytest.mark.parametrize('code, ref', [
    (key, value['ref']) for key, value in ALT_PATTERNS.items() if 'pattern' in value and 'ref' in value
])
def test_alt_pattern(code, ref, count):
    number = PhoneNumber(code).get_number()
    num_obj = phonenumbers.parse(number, code)
    assert phonenumbers.is_valid_number_for_region(num_obj, ref)


@pytest.mark.phonenumbers
@pytest.mark.parametrize('count', range(5))
@pytest.mark.parametrize('code, ref', [
    (key, value['ref']) for key, value in ALT_PATTERNS.items() if 'pattern' not in value and 'ref' in value
])
def test_ref(code, ref, count):
    number = PhoneNumber(code).get_number()
    num_obj = phonenumbers.parse(number, code)
    assert phonenumbers.is_valid_number_for_region(num_obj, ref)



import argparse
import logging
import re
from pathlib import Path
from subprocess import run as cmd
from typing import Final

import requests

from dev_tools.patterns_generator import get_latest, parsing_version, main

ROOT_DIR: Final[Path] = Path(__file__).absolute().parent.parent
logging.basicConfig(format='[%(asctime)s] %(levelname)s:%(message)s', level=logging.INFO)


def arg_parser() -> argparse.Namespace:
    parser = argparse.ArgumentParser(
        prog="update",
        add_help=True,
        description="Update phone-gen patterns",
    )
    parser.add_argument(
        "-u", "--update", dest="update", help="Need to update?", action='store_true'
    )
    parser.add_argument(
        "-p", "--ignore-phonenumbers", dest="ignore_phonenumbers", help="Ignore phonenumbers versions",
        action='store_true'
    )
    parser.add_argument(
        "-b", "--no-black", dest="no_black", help="Do not run black", action='store_true'
    )
    parser.add_argument(
        "-f", "--no-flake", dest="no_flake", help="Do not run flake8", action='store_true'
    )
    parser.add_argument(
        "-t", "--no-tests", dest="no_tests", help="Do not run tests", action='store_true'
    )
    return parser.parse_args()


def is_need_update(version: str) -> bool:
    if (pattern_file := ROOT_DIR / 'phone_gen' / 'patterns.py').exists():
        with pattern_file.open('r') as _file:
            file = _file.read()
            if match := re.findall(r'libphonenumber v([\d.]+)"', file):
                return version != match[0]
    return True


def check_phonenumbers_libs(version: str) -> bool:
    response = requests.get('https://pypi.org/pypi/phonenumbers/json')
    response.raise_for_status()
    releases = response.json()['releases']
    return version in releases.keys()


def update_pipfile(version: str) -> None:
    logging.info('Update Pipfile')
    if (pipfile_path := ROOT_DIR / 'Pipfile').exists():
        with pipfile_path.open('r') as _file:
            pipfile = _file.read()
        pipfile = re.sub(r'phonenumbers\s?=\s?"==[\d.]+"', f'phonenumbers = "=={version}"', pipfile)
        with pipfile_path.open('w') as _file:
            _file.write(pipfile)
        return
    logging.critical('Not found Pipfile')
    exit(-1)


def update_workflow(version: str) -> None:
    logging.debug('Update workflow')
    if (workflow_path := ROOT_DIR / '.github' / 'workflows' / 'python-package.yml').exists():
        with workflow_path.open('r') as _file:
            workflow = _file.read()
        workflow = re.sub(r'phonenumbers==[\d.]+', f'phonenumbers=={version}', workflow)
        with workflow_path.open('w') as _file:
            _file.write(workflow)
        return
    logging.critical('Not found workflow file')
    exit(-1)


def run(args: argparse.Namespace) -> None:
    version = parsing_version(get_latest())
    if not args.update and not is_need_update(version=version):
        logging.info('No update required')
        exit(0)
    is_phonenumbers = check_phonenumbers_libs(version=version)
    if not is_phonenumbers and not args.ignore_phonenumbers:
        logging.critical(f'No actual version phonenumbers. {version}')
        exit(-1)
    if is_phonenumbers:
        update_pipfile(version=version)
        update_workflow(version=version)

    logging.info('Update requirements')
    cmd(['pipenv', 'update', '-d'], cwd=ROOT_DIR)

    logging.info('Update patterns')
    main(patterns_tag=f'v{version}')

    if not args.no_black:
        logging.info('Run black')
        cmd(['black', 'phone_gen/'], cwd=ROOT_DIR)
    if not args.no_flake:
        logging.info('Run flake8')
        cmd(['flake8', 'phone_gen/'], cwd=ROOT_DIR)
    if not args.no_tests:
        logging.info('Run tests')
        cmd(['pytest', 'tests/'], cwd=ROOT_DIR)
    logging.info('Done')


if __name__ == '__main__':
    run(args=arg_parser())



import os
import random

import phonenumbers
import pytest

from phone_gen.patterns import PATTERNS


@pytest.mark.phonenumbers
@pytest.mark.parametrize('country', random.sample(tuple(PATTERNS['data'].keys()), 20))
@pytest.mark.parametrize('arg', ('', '-n ', '-m '))
def test_get_country(capfd, country, arg):
    os.system(f'phone-gen {arg}{country}')
    captured = capfd.readouterr()
    num_obg = phonenumbers.parse(captured.out, country)
    assert phonenumbers.is_valid_number_for_region(num_obg, country)


@pytest.mark.phonenumbers
@pytest.mark.parametrize('country', random.sample(tuple(PATTERNS['data'].keys()), 20))
@pytest.mark.parametrize('arg', ('', 'n', 'm'))
def test_get_without_country_code(capfd, country, arg):
    os.system(f'phone-gen -f{arg} {country}')
    captured = capfd.readouterr()
    code = PATTERNS['data'][country]['code']
    num_obg = phonenumbers.parse(f'{code}{captured.out}', country)
    assert phonenumbers.is_valid_number_for_region(num_obg, country)


def test_invalid_country(capfd):
    os.system('phone-gen qwe')
    captured = capfd.readouterr()
    assert captured.out.strip() == 'Error: Not found country "qwe"'


@pytest.mark.phonenumbers
@pytest.mark.parametrize('country_name, code', (('Germany', 'DE'), ('Panama', 'PA'), ('Turkey', 'TR')))
@pytest.mark.parametrize('arg', ('', '-n ', '-m '))
def test_find_country_name(capfd, country_name, code, arg):
    os.system(f'phone-gen {arg}{country_name}')
    captured = capfd.readouterr()
    num_obg = phonenumbers.parse(captured.out, code)
    assert phonenumbers.is_valid_number_for_region(num_obg, code)


@pytest.mark.phonenumbers
@pytest.mark.parametrize('iso, code', (('VNM', 'VN'), ('SLE', 'SL'), ('MCO', 'MC')))
@pytest.mark.parametrize('arg', ('', '-n ', '-m '))
def test_find_iso3(capfd, iso, code, arg):
    os.system(f'phone-gen {arg}{iso}')
    captured = capfd.readouterr()
    num_obg = phonenumbers.parse(captured.out, code)
    assert phonenumbers.is_valid_number_for_region(num_obg, code)



from pathlib import Path

from setuptools import find_packages, setup

readme = Path(".", "README.md").absolute()
with readme.open("r", encoding="utf-8") as file:
    long_description = file.read()

setup(
    name="phone_gen",
    packages=find_packages(exclude=("tests", "dev_tools")),
    url="https://github.com/tolstislon/phone-gen",
    license="MIT License",
    author="tolstislon",
    author_email="tolstislon@gmail.com",
    description="International phone number generation",
    long_description=long_description,
    long_description_content_type="text/markdown",
    use_scm_version={"write_to": "phone_gen/__version__.py"},
    setup_requires=["setuptools_scm"],
    entry_points={
        "console_scripts": ["phone-gen=phone_gen.cli:main"],
    },
    python_requires=">=3.7",
    include_package_data=True,
    keywords=["testing", "test-data", "phone-number", "phone", "test-data-generator"],
    classifiers=[
        "Development Status :: 5 - Production/Stable",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3.10",
        "Programming Language :: Python :: 3.11",
        "Programming Language :: Python :: 3.12",
        "Programming Language :: Python :: Implementation :: CPython",
        "Programming Language :: Python :: Implementation :: PyPy",
        "Topic :: Software Development :: Testing",
        "Topic :: Communications :: Telephony",
    ],
)



import random

import phonenumbers
import pytest

from phone_gen import load_alt_patters, clean_alt_patters, PhoneNumber
from phone_gen.patterns import PATTERNS


@pytest.fixture(params=random.sample(tuple(PATTERNS['data'].keys()), 20))
def load_fixture(request):
    load_alt_patters({
        request.param: {
            "code": "0",
            "pattern": "([2]{7})",
            "mobile": "([1]{9})"
        }
    })
    yield request.param
    clean_alt_patters()


@pytest.mark.phonenumbers
def test_load_alt_patterns(load_fixture):
    phone_number = PhoneNumber(load_fixture)
    assert phone_number.get_national() == '+0{}'.format('2' * 7)
    assert phone_number.get_mobile() == '+0{}'.format('1' * 9)
    clean_alt_patters()
    phone_number = PhoneNumber(load_fixture)
    num_obj = phonenumbers.parse(phone_number.get_national(), load_fixture)
    assert phonenumbers.is_valid_number_for_region(num_obj, load_fixture)
    num_obj = phonenumbers.parse(phone_number.get_mobile(), load_fixture)
    assert phonenumbers.is_valid_number_for_region(num_obj, load_fixture)
